searchData={"items":[{"type":"module","title":"ExStan","doc":"**ExStan** is a Elixir interface to Stan, a package for Bayesian inference.\n\nStan® is a state-of-the-art platform for statistical modeling and\nhigh-performance statistical computation. Thousands of users rely on Stan for\nstatistical modeling, data analysis, and prediction in the social, biological,\nand physical sciences, engineering, and business.\n\nThis project is primarily based on the [PyStan](https://github.com/stan-dev/pystan) repository.","ref":"ExStan.html"},{"type":"function","title":"ExStan.build/3","doc":"Builds (compiles) a Stan program.","ref":"ExStan.html#build/3"},{"type":"function","title":"Parameters - ExStan.build/3","doc":"- `program_code`: Stan program code describing a Stan model.\n- `data`: An Elixir map providing the data for the model. Variable names are the keys and the values are their associated values. Default is an empty map, suitable for Stan programs with no `data` block.\n- `random_seed`: Random seed, a positive integer for random number generation. Used to ensure that results can be reproduced.","ref":"ExStan.html#build/3-parameters"},{"type":"function","title":"Returns - ExStan.build/3","doc":"- `Model`: an instance of Model","ref":"ExStan.html#build/3-returns"},{"type":"function","title":"Notes - ExStan.build/3","doc":"C++ reserved words and Stan reserved words may not be used for variable names; see the Stan User's Guide for a complete list.","ref":"ExStan.html#build/3-notes"},{"type":"module","title":"ExStan.Client","doc":"Lightweight client for interacting with the HTTPStan server.","ref":"ExStan.Client.html"},{"type":"function","title":"ExStan.Client.delete/1","doc":"Sends a DELETE request to the specified path.","ref":"ExStan.Client.html#delete/1"},{"type":"function","title":"ExStan.Client.get/1","doc":"Sends a GET request to the specified path.","ref":"ExStan.Client.html#get/1"},{"type":"function","title":"ExStan.Client.post/2","doc":"Sends a POST request to the specified path with the given body.","ref":"ExStan.Client.html#post/2"},{"type":"module","title":"ExStan.Constants","doc":"Constants used in the ExStan library.\nThis is a simplified version of the arguments module implemented in HTTPStan.\nThe argument types are included in the priv directory under the filename \"cmdstan-help-all.json\"","ref":"ExStan.Constants.html"},{"type":"function","title":"ExStan.Constants.default_sample_num_flat/0","doc":"","ref":"ExStan.Constants.html#default_sample_num_flat/0"},{"type":"function","title":"ExStan.Constants.default_sample_num_samples/0","doc":"","ref":"ExStan.Constants.html#default_sample_num_samples/0"},{"type":"function","title":"ExStan.Constants.default_sample_num_thin/0","doc":"","ref":"ExStan.Constants.html#default_sample_num_thin/0"},{"type":"function","title":"ExStan.Constants.default_sample_num_warmup/0","doc":"","ref":"ExStan.Constants.html#default_sample_num_warmup/0"},{"type":"function","title":"ExStan.Constants.default_sample_save_warmup/0","doc":"","ref":"ExStan.Constants.html#default_sample_save_warmup/0"},{"type":"module","title":"ExStan.Fit","doc":"Stores draws from one or more chains.\n\nReturned by methods of a `Model`. Users will not instantiate this class directly.\n\nA `Fit` instance provides a user-friendly view of draws, accessible via the `to_frame` method.","ref":"ExStan.Fit.html"},{"type":"function","title":"ExStan.Fit.new/1","doc":"Creates a new `Fit` struct, given the specified options.\nIt will validate the options and parse the draws from the Stan outputs.","ref":"ExStan.Fit.html#new/1"},{"type":"function","title":"ExStan.Fit.to_frame/1","doc":"Converts the draws from a `Fit` struct into a data frame.\nThis function iterates over the draws and feature names individually, and will get slower as the number of draws/chains increases.","ref":"ExStan.Fit.html#to_frame/1"},{"type":"function","title":"Parameters - ExStan.Fit.to_frame/1","doc":"- `%Fit{}`: A `Fit` struct containing the draws and feature names.","ref":"ExStan.Fit.html#to_frame/1-parameters"},{"type":"function","title":"Returns - ExStan.Fit.to_frame/1","doc":"A `DataFrame` object with the draws as rows and feature names with chain and sample indexes as columns.","ref":"ExStan.Fit.html#to_frame/1-returns"},{"type":"function","title":"Errors - ExStan.Fit.to_frame/1","doc":"- Raises an error if the `Explorer` module is not available.","ref":"ExStan.Fit.html#to_frame/1-errors"},{"type":"module","title":"ExStan.Model","doc":"Stores data associated with a Stan model and proxies calls to Stan services.\n\nReturned by `ExStan.build`. Users will not instantiate this class directly.","ref":"ExStan.Model.html"},{"type":"function","title":"ExStan.Model.constrain_pars/4","doc":"Calculate the log probability of a set of unconstrained parameters.\n\nArguments:\n    model: The model containing the data.\n    unconstrained_parameters: A sequence of unconstrained parameters.\n    include_tparams: Apply jacobian adjust transform for transformed parameters.\n    include_gqs: Apply jacobian adjust transform for generated quantities.\n\nReturns:\n    The log probability of the unconstrained parameters.\n\nNotes:\n    The unconstrained parameters are passed to the log_prob\n    function in stan::model.","ref":"ExStan.Model.html#constrain_pars/4"},{"type":"function","title":"ExStan.Model.fixed_param/2","doc":"Draws samples from the model using `stan::services::sample::fixed_param`.\n\nParameters in `opts` will be passed to `stan::services::sample::fixed_param`. Parameter names are\nidentical to those used in CmdStan. See the CmdStan documentation for\nparameter descriptions and default values.\n\n`num_chains` is an ExStan-specific argument indicating the number of\nindependent processes to use when drawing samples.\n\nReturns:\n- `%ExStan.Fit{}`: An instance of `%ExStan.Fit{}` allowing access to draws.","ref":"ExStan.Model.html#fixed_param/2"},{"type":"function","title":"ExStan.Model.grad_log_prob/2","doc":"Calculate the gradient of the log posterior evaluated at the unconstrained parameters.\n\nArguments:\n    model: The model containing the data.\n    unconstrained_parameters: A sequence of unconstrained parameters.\n\nReturns:\n    The gradient of the log posterior evaluated at the unconstrained parameters.\n\nNotes:\n    The unconstrained parameters are passed to the log_prob_grad\n    function in stan::model.","ref":"ExStan.Model.html#grad_log_prob/2"},{"type":"function","title":"ExStan.Model.hmc_nuts_diag_e_adapt/2","doc":"Draws samples from the model using `stan::services::sample::hmc_nuts_diag_e_adapt`.\n\nParameters in `opts` will be passed to `stan::services::sample::hmc_nuts_diag_e_adapt`. Parameter names are\nidentical to those used in CmdStan. See the CmdStan documentation for\nparameter descriptions and default values.\n\nThere is one exception: `num_chains`. `num_chains` is an\nExStan-specific keyword argument. It indicates the number of\nindependent processes to use when drawing samples.","ref":"ExStan.Model.html#hmc_nuts_diag_e_adapt/2"},{"type":"function","title":"Returns - ExStan.Model.hmc_nuts_diag_e_adapt/2","doc":"- `%ExStan.Fit{}`: An instance of `%ExStan.Fit{}` allowing access to draws.","ref":"ExStan.Model.html#hmc_nuts_diag_e_adapt/2-returns"},{"type":"function","title":"ExStan.Model.log_prob/3","doc":"Calculate the log probability of a set of unconstrained parameters.\n\nArguments:\n    model: The model containing the data.\n    unconstrained_parameters: A sequence of unconstrained parameters.\n    adjust_transform: Apply jacobian adjust transform.\n\nReturns:\n    The log probability of the unconstrained parameters.\n\nNotes:\n    The unconstrained parameters are passed to the log_prob\n    function in stan::model.","ref":"ExStan.Model.html#log_prob/3"},{"type":"function","title":"ExStan.Model.sample/2","doc":"Draws samples from the model. For easy reference, documentation is copied from Pystan.\n\nParameters in `params` will be passed to the default sample function.\nThe default sample function is currently `stan::services::sample::hmc_nuts_diag_e_adapt`.\nParameter names are identical to those used in CmdStan. See the CmdStan documentation for\nparameter descriptions and default values.\n\nThe function also accepts all options supported by CmdStand. Specifically, `num_chains` indicates the number of independent processes\nto use when drawing samples.","ref":"ExStan.Model.html#sample/2"},{"type":"function","title":"Returns - ExStan.Model.sample/2","doc":"- Fit: instance of Fit allowing access to draws.","ref":"ExStan.Model.html#sample/2-returns"},{"type":"function","title":"Examples - ExStan.Model.sample/2","doc":"User-defined initial values for parameters must be provided\nfor each chain. Typically they will be the same for each chain.\nThe following example shows how user-defined initial parameters\nare provided:\n\n```\nprogram_code = \"parameters {real y;} model {y ~ normal(0,1);}\"\nposterior = ExStan.build(program_code)\nfit = ExStan.Model.sample(posterior, num_chains: 2, init: [{\"y\": 3}, {\"y\": 3}])\n```","ref":"ExStan.Model.html#sample/2-examples"},{"type":"function","title":"ExStan.Model.unconstrain_pars/2","doc":"Reads constrained parameter values from their specified context and returns a\nsequence of unconstrained parameter values.\n\nArguments:\n    model: The model containing the data.\n    constrained_parameters: Constrained parameter values and their specified context\n\nReturns:\n    A sequence of unconstrained parameters.\n\nNotes:\n    The unconstrained parameters are passed to the `transform_inits` method of the\n    `model_base` instance. See `model_base.hpp` in the Stan C++ library for details.","ref":"ExStan.Model.html#unconstrain_pars/2"},{"type":"extras","title":"Iris: Linear Regression","doc":"<!-- livebook:{\"file_entries\":[{\"name\":\"iris.png\",\"type\":\"attachment\"}],\"persist_outputs\":true} -->\n\n# Iris: Linear Regression\n\n```elixir\nMix.install([\n  {:ex_stan, path: \"~/shubham/ex_stan\"},\n  {:explorer, \"~> 0.8.0\"},\n  {:kino_explorer, \"~> 0.1.18\"},\n  {:vega_lite, \"~> 0.1.6\"},\n  {:kino_vega_lite, \"~> 0.1.10\"},\n  {:scidata, \"~> 0.1.8\"}\n])\n```","ref":"iris_linear_regression.html"},{"type":"extras","title":"Introduction - Iris: Linear Regression","doc":"Here, we will aim to perform simple linear regression using Stan. We'll also look at performing simple checks to evaluate if our model is correct.\n\n```elixir\nalias ExStan.{Model, Fit}\nrequire Explorer.DataFrame, as: DF\nalias VegaLite, as: Vl\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nVegaLite\n```\n\n![](files/iris.png)\n\n<!-- livebook:{\"break_markdown\":true} -->\n\nThe dataset consists of 150 records, with 50 records per class of flowers. It describes the length and width of sepals and petals for each class.\n\nSuppose we want to model the relationship between the sepal length and the petal length for the Setosa class of flowers i.e we want to measure the changes to petal length as the sepal length changes.\n\nFor the sake of simplicity, we will assume that this is a linear relationship.\n\nLinear Regression has the following form:\n$$$\ny = \\alpha + \\beta x + \\epsilon\n$$$\n\n* **x** referes to the input variable\n* **y** refers to the output variable\n* **$\\beta$** refers to the slope of the line\n* **$\\alpha$** represents the y-intercept for the line i.e the value of y when x = 0\n* **$\\epsilon$** is the irreducible random noise associated with the output y. This is drawn from a gaussian distribution with mean 0 and variance $\\sigma$. This  has the form:\n  $$\n  \\epsilon \\sim \\mathcal{N}(0, \\sigma)\n  $$\n\n**Key Assumption**\n\nWe assume that $y$ scatters around the linear line $\\alpha + \\beta x$ i.e the distribution of the $y$ given the parameters and input is a gaussian distribution with mean equal to the line and variance(scale) equal to $\\sigma$\n\n$$\nP(y|x,\\alpha,\\beta,\\sigma) = \\mathcal{N}(y|\\alpha + \\beta x, \\sigma)\n$$\n\nWe will use bayesian inference to find all the lines and scatters consistent with the observed data from the posterior, which can be obtained from Bayes Rule as\n\n$$\nP(\\alpha, \\beta, \\sigma |y, x) \\propto P(y|x,\\alpha,\\beta,\\sigma) P(\\alpha, \\beta, \\sigma, x)\n$$","ref":"iris_linear_regression.html#introduction"},{"type":"extras","title":"Dataset - Iris: Linear Regression","doc":"```elixir\n{train_features, train_labels} = Scidata.Iris.download()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{[\n   [5.1, 3.5, 1.4, 0.2],\n   [4.9, 3.0, 1.4, 0.2],\n   [4.7, 3.2, 1.3, 0.2],\n   [4.6, 3.1, 1.5, 0.2],\n   [5.0, 3.6, 1.4, 0.2],\n   [5.4, 3.9, 1.7, 0.4],\n   [4.6, 3.4, 1.4, 0.3],\n   [5.0, 3.4, 1.5, 0.2],\n   [4.4, 2.9, 1.4, 0.2],\n   [4.9, 3.1, 1.5, 0.1],\n   [5.4, 3.7, 1.5, 0.2],\n   [4.8, 3.4, 1.6, 0.2],\n   [4.8, 3.0, 1.4, 0.1],\n   [4.3, 3.0, 1.1, 0.1],\n   [5.8, 4.0, 1.2, 0.2],\n   [5.7, 4.4, 1.5, 0.4],\n   [5.4, 3.9, 1.3, 0.4],\n   [5.1, 3.5, 1.4, 0.3],\n   [5.7, 3.8, 1.7, 0.3],\n   [5.1, 3.8, 1.5, 0.3],\n   [5.4, 3.4, 1.7, 0.2],\n   [5.1, 3.7, 1.5, 0.4],\n   [4.6, 3.6, 1.0, 0.2],\n   [5.1, 3.3, 1.7, 0.5],\n   [4.8, 3.4, 1.9, 0.2],\n   [5.0, 3.0, 1.6, 0.2],\n   [5.0, 3.4, 1.6, 0.4],\n   [5.2, 3.5, 1.5, 0.2],\n   [5.2, 3.4, 1.4, 0.2],\n   [4.7, 3.2, 1.6, 0.2],\n   [4.8, 3.1, 1.6, 0.2],\n   [5.4, 3.4, 1.5, 0.4],\n   [5.2, 4.1, 1.5, 0.1],\n   [5.5, 4.2, 1.4, 0.2],\n   [4.9, 3.1, 1.5, 0.1],\n   [5.0, 3.2, 1.2, 0.2],\n   [5.5, 3.5, 1.3, 0.2],\n   [4.9, 3.1, 1.5, 0.1],\n   [4.4, 3.0, 1.3, 0.2],\n   [5.1, 3.4, 1.5, 0.2],\n   [5.0, 3.5, 1.3, 0.3],\n   [4.5, 2.3, 1.3, 0.3],\n   [4.4, 3.2, 1.3, 0.2],\n   [5.0, 3.5, 1.6, 0.6],\n   [5.1, 3.8, 1.9, 0.4],\n   [4.8, 3.0, 1.4, ...],\n   [5.1, 3.8, ...],\n   [4.6, ...],\n   [...],\n   ...\n ],\n [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,\n  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, ...]}\n```\n\n```elixir\nfeature_names = [\"sepal_length\", \"sepal_width\", \"petal_length\", \"petal_width\"]\n\ndf =\n  train_features\n  |> Enum.with_index(fn x, idx ->\n    Enum.zip(feature_names, x) ++ [{\"class\", Enum.at(train_labels, idx)}]\n  end)\n  |> DF.new()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```text\n#Explorer.DataFrame \n```\n\n```elixir\nvalues =\n  df\n  |> DF.filter(class == 1)\n  |> DF.select([\"sepal_length\", \"petal_length\"])\n  |> DF.to_rows()\n\nVl.new(width: 600, height: 300, title: \"Versicolor Sepal Length vs Petal Length\")\n|> Vl.data_from_values(values)\n|> Vl.encode_field(:x, \"sepal_length\", type: :quantitative, scale: [domain: [0.0, 7.5]])\n|> Vl.encode_field(:y, \"petal_length\", type: :quantitative, scale: [domain: [0.0, 6.5]])\n|> Vl.layers([\n  Vl.new()\n  |> Vl.mark(:point)\n  |> Vl.encode_field(:y, \"petal_length\", type: :quantitative)\n])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n\n\n```elixir\nx =\n  df\n  |> DF.filter(class == 1)\n  |> DF.select([\"sepal_length\"])\n  |> DF.to_columns()\n  |> Map.get(\"sepal_length\")\n\ny =\n  df\n  |> DF.filter(class == 1)\n  |> DF.select([\"petal_length\"])\n  |> DF.to_columns()\n  |> Map.get(\"petal_length\")\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[4.7, 4.5, 4.9, 4.0, 4.6, 4.5, 4.7, 3.3, 4.6, 3.9, 3.5, 4.2, 4.0, 4.7, 3.6, 4.4, 4.5, 4.1, 4.5, 3.9,\n 4.8, 4.0, 4.9, 4.7, 4.3, 4.4, 4.8, 5.0, 4.5, 3.5, 3.8, 3.7, 3.9, 5.1, 4.5, 4.5, 4.7, 4.4, 4.1, 4.0,\n 4.4, 4.6, 4.0, 3.3, 4.2, 4.2, 4.2, 4.3, 3.0, 4.1]\n```","ref":"iris_linear_regression.html#dataset"},{"type":"extras","title":"Model - Iris: Linear Regression","doc":"The key part to encode as in the model is the expert knowledge into the prior distributions of the variable.\n\nIf we are confident that the value for $\\alpha$ is normally distributed with mean 3 and variance 1, we can represent the prior for $alpha$ as $\\mathcal{N}(3,1)$.\n\nLet's assume that we don't have an expert knowledge about the parameters. In this case, we will assign \"weakly informative\" priors to the variables to increase the changes of finding the right parameter. Eg: Use a super-wide normal prior density.\n\n```elixir\nmodel_code = ~S\"\"\"\ndata {\n  int  N; // number of points\n  vector[N] x; // sepal length i.e covariate\n  vector[N] y; // petal length i.e variate\n}\n\nparameters {\n  real alpha; // intercept\n  real beta; // slope\n  real  sigma; //scatter\n}\n\nmodel {\n  // priors\n  alpha ~ normal(0, 10);\n  beta ~ normal(0, 10);\n  sigma ~ normal(0, 1);\n\n  // likelihood\n  y ~ normal(alpha + beta * x, sigma); \n}\n\"\"\"\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\"data {\\n  int  N; // number of points\\n  vector[N] x; // sepal length i.e covariate\\n  vector[N] y; // petal length i.e variate\\n}\\n\\nparameters {\\n  real alpha; // intercept\\n  real beta; // slope\\n  real  sigma; //scatter\\n}\\n\\nmodel {\\n  // priors\\n  alpha ~ normal(0, 10);\\n  beta ~ normal(0, 10);\\n  sigma ~ normal(0, 1);\\n\\n  // likelihood\\n  y ~ normal(alpha + beta * x, sigma); \\n}\\n\"\n```\n\n```elixir\ndata = %{\n  \"x\" => x,\n  \"y\" => y,\n  \"N\" => length(x)\n}\n\nmodel = ExStan.build(model_code, data)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n20:08:07.064 [info] Building model..\n\n20:08:07.064 [debug] POST http://localhost:8080/v1/models\n\n20:08:07.080 [info] Model created\n\n20:08:07.080 [info] Messages from stanc: \n\n20:08:07.080 [debug] POST http://localhost:8080/v1/models/d2ncp7rn/params\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%ExStan.Model{\n  model_name: \"models/d2ncp7rn\",\n  program_code: \"data {\\n  int  N; // number of points\\n  vector[N] x; // sepal length i.e covariate\\n  vector[N] y; // petal length i.e variate\\n}\\n\\nparameters {\\n  real alpha; // intercept\\n  real beta; // slope\\n  real  sigma; //scatter\\n}\\n\\nmodel {\\n  // priors\\n  alpha ~ normal(0, 10);\\n  beta ~ normal(0, 10);\\n  sigma ~ normal(0, 1);\\n\\n  // likelihood\\n  y ~ normal(alpha + beta * x, sigma); \\n}\\n\",\n  data: %{\n    \"N\" => 50,\n    \"x\" => [7.0, 6.4, 6.9, 5.5, 6.5, 5.7, 6.3, 4.9, 6.6, 5.2, 5.0, 5.9, 6.0, 6.1, 5.6, 6.7, 5.6,\n     5.8, 6.2, 5.6, 5.9, 6.1, 6.3, 6.1, 6.4, 6.6, 6.8, 6.7, 6.0, 5.7, 5.5, 5.5, 5.8, 6.0, 5.4, 6.0,\n     6.7, 6.3, 5.6, 5.5, 5.5, 6.1, 5.8, 5.0, 5.6, ...],\n    \"y\" => [4.7, 4.5, 4.9, 4.0, 4.6, 4.5, 4.7, 3.3, 4.6, 3.9, 3.5, 4.2, 4.0, 4.7, 3.6, 4.4, 4.5,\n     4.1, 4.5, 3.9, 4.8, 4.0, 4.9, 4.7, 4.3, 4.4, 4.8, 5.0, 4.5, 3.5, 3.8, 3.7, 3.9, 5.1, 4.5, 4.5,\n     4.7, 4.4, 4.1, 4.0, 4.4, 4.6, 4.0, 3.3, ...]\n  },\n  param_names: [\"alpha\", \"beta\", \"sigma\"],\n  constrained_param_names: [[\"alpha\"], [\"beta\"], [\"sigma\"]],\n  dims: [[], [], []],\n  random_seed: nil\n}\n```","ref":"iris_linear_regression.html#model"},{"type":"extras","title":"Inference - Iris: Linear Regression","doc":"```elixir\nfit = Model.sample(model, num_chains: 4, num_warmup: 1000, num_samples: 2000, stepsize: 0.1)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n13:08:44.398 [info] `chain` id is set automatically.\n\n13:08:44.398 [debug] POST http://localhost:8080/v1/models/d2ncp7rn/fits\n\n13:08:44.407 [debug] POST http://localhost:8080/v1/models/d2ncp7rn/fits\n\n13:08:44.433 [debug] POST http://localhost:8080/v1/models/d2ncp7rn/fits\n\n13:08:44.808 [debug] POST http://localhost:8080/v1/models/d2ncp7rn/fits\n\n13:08:45.039 [debug] GET http://localhost:8080/v1/operations/iwyj7bq6\n\n13:08:45.271 [debug] GET http://localhost:8080/v1/operations/kadv2z6x\n\n13:08:45.272 [debug] GET http://localhost:8080/v1/operations/wkda3ztn\n\n13:08:45.273 [debug] GET http://localhost:8080/v1/operations/xmvjrcyr\n\n13:08:45.274 [info] Sampling: 100%, done.\n\n13:08:45.274 [debug] GET http://localhost:8080/v1/models/d2ncp7rn/fits/iwyj7bq6\n\n13:08:45.288 [debug] DELETE http://localhost:8080/v1/models/d2ncp7rn/fits/iwyj7bq6\n\n13:08:45.289 [debug] GET http://localhost:8080/v1/models/d2ncp7rn/fits/kadv2z6x\n\n13:08:45.302 [debug] DELETE http://localhost:8080/v1/models/d2ncp7rn/fits/kadv2z6x\n\n13:08:45.304 [debug] GET http://localhost:8080/v1/models/d2ncp7rn/fits/wkda3ztn\n\n13:08:45.330 [debug] DELETE http://localhost:8080/v1/models/d2ncp7rn/fits/wkda3ztn\n\n13:08:45.333 [debug] GET http://localhost:8080/v1/models/d2ncp7rn/fits/xmvjrcyr\n\n13:08:45.343 [debug] DELETE http://localhost:8080/v1/models/d2ncp7rn/fits/xmvjrcyr\n\n13:08:45.362 [info] Messages received during sampling:\n\n13:08:45.363 [info]   Gradient evaluation took 1.4e-05 seconds\n\n13:08:45.363 [info]   1000 transitions using 10 leapfrog steps per transition would take 0.14 seconds.\n\n13:08:45.363 [info]   Adjust your expectations accordingly!\n\n13:08:45.363 [info]   Gradient evaluation took 1.4e-05 seconds\n\n13:08:45.363 [info]   1000 transitions using 10 leapfrog steps per transition would take 0.14 seconds.\n\n13:08:45.363 [info]   Adjust your expectations accordingly!\n\n13:08:45.363 [info]   Gradient evaluation took 3e-06 seconds\n\n13:08:45.363 [info]   1000 transitions using 10 leapfrog steps per transition would take 0.03 seconds.\n\n13:08:45.363 [info]   Adjust your expectations accordingly!\n\n13:08:45.363 [info]   Gradient evaluation took 2e-06 seconds\n\n13:08:45.363 [info]   1000 transitions using 10 leapfrog steps per transition would take 0.02 seconds.\n\n13:08:45.363 [info]   Adjust your expectations accordingly!\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%ExStan.Fit{\n  stan_outputs: [\"{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Gradient evaluation took 1.4e-05 seconds\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:1000 transitions using 10 leapfrog steps per transition would take 0.14 seconds.\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Adjust your expectations accordingly!\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:    1 / 3000 [  0%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  100 / 3000 [  3%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  200 / 3000 [  6%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  300 / 3000 [ 10%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  400 / 3000 [ 13%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  500 / 3000 [ 16%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  600 / 3000 [ 20%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  700 / 3000 [ 23%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  800 / 3000 [ 26%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  900 / 3000 [ 30%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1000 / 3000 [ 33%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1001 / 3000 [ 33%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1100 / 3000 [ 36%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1200 / 3000 [ 40%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1300 / 3000 [ 43%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1400 / 3000 [ 46%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1500 / 3000 [ 50%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1600 / 3000 [ 53%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1700 / 3000 [ 56%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1800 / 3000 [ 60%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1900 / 3000 [ 63%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2000 / 3000 [ 66%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2100 / 3000 [ 70%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2200 / 3000 [ 73%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2300 / 3000 [ 76%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2400 / 3000 [ 80%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2500 / 3000 [ 83%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2600 / 3000 [ 86%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2700 / 3000 [ 90%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2800 / 3000 [ 93%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2900 / 3000 [ 96%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 3000 / 3000 [100%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info: Elapsed Time: 0.092 seconds (Warm-up)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.286 seconds (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.378 seconds (Total)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"initialization\\\",\\\"values\\\":[-0.9783659596645611,-0.5573192648261118,1.4981911745129347]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Adaptation terminated\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Step size = 0.0576725\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Diagonal elements of inverse mass matrix:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"0.236142, 0.006762\" <> ...,\n   \"{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Gradient evaluation took 1.4e-05 seconds\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:1000 transitions using 10 leapfrog steps per transition would take 0.14 seconds.\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Adjust your expectations accordingly!\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:    1 / 3000 [  0%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  100 / 3000 [  3%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  200 / 3000 [  6%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  300 / 3000 [ 10%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  400 / 3000 [ 13%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  500 / 3000 [ 16%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  600 / 3000 [ 20%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  700 / 3000 [ 23%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  800 / 3000 [ 26%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  900 / 3000 [ 30%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1000 / 3000 [ 33%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1001 / 3000 [ 33%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1100 / 3000 [ 36%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1200 / 3000 [ 40%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1300 / 3000 [ 43%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1400 / 3000 [ 46%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1500 / 3000 [ 50%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1600 / 3000 [ 53%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1700 / 3000 [ 56%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1800 / 3000 [ 60%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1900 / 3000 [ 63%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2000 / 3000 [ 66%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2100 / 3000 [ 70%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2200 / 3000 [ 73%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2300 / 3000 [ 76%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2400 / 3000 [ 80%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2500 / 3000 [ 83%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2600 / 3000 [ 86%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2700 / 3000 [ 90%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2800 / 3000 [ 93%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2900 / 3000 [ 96%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 3000 / 3000 [100%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info: Elapsed Time: 0.076 seconds (Warm-up)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.13 seconds (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.206 seconds (Total)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"initialization\\\",\\\"values\\\":[0.05442034671090079,-1.4899873790046734,0.10489923554534775]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Adaptation terminated\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Step size = 0.063929\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Diagonal elements of inverse mass matrix:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"0.304065, 0.0084158\" <> ...,\n   \"{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Gradient evaluation took 3e-06 seconds\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:1000 transitions using 10 leapfrog steps per transition would take 0.03 seconds.\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Adjust your expectations accordingly!\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:    1 / 3000 [  0%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  100 / 3000 [  3%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  200 / 3000 [  6%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  300 / 3000 [ 10%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  400 / 3000 [ 13%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  500 / 3000 [ 16%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  600 / 3000 [ 20%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  700 / 3000 [ 23%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  800 / 3000 [ 26%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  900 / 3000 [ 30%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1000 / 3000 [ 33%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1001 / 3000 [ 33%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1100 / 3000 [ 36%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1200 / 3000 [ 40%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1300 / 3000 [ 43%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1400 / 3000 [ 46%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1500 / 3000 [ 50%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1600 / 3000 [ 53%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1700 / 3000 [ 56%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1800 / 3000 [ 60%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1900 / 3000 [ 63%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2000 / 3000 [ 66%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2100 / 3000 [ 70%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2200 / 3000 [ 73%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2300 / 3000 [ 76%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2400 / 3000 [ 80%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2500 / 3000 [ 83%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2600 / 3000 [ 86%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2700 / 3000 [ 90%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2800 / 3000 [ 93%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2900 / 3000 [ 96%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 3000 / 3000 [100%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info: Elapsed Time: 0.057 seconds (Warm-up)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.128 seconds (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.185 seconds (Total)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"initialization\\\",\\\"values\\\":[-0.2951205304732387,-0.9470579910310857,0.20042005425138632]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Adaptation terminated\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Step size = 0.0590723\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Diagonal elements of inverse mass matrix:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"0.357472, 0.0101886\" <> ...,\n   \"{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Gradient evaluation took 2e-06 seconds\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:1000 transitions using 10 leapfrog steps per transition would take 0.02 seconds.\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Adjust your expectations accordingly!\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:    1 / 3000 [  0%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  100 / 3000 [  3%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  200 / 3000 [  6%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  300 / 3000 [ 10%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  400 / 3000 [ 13%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  500 / 3000 [ 16%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  600 / 3000 [ 20%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  700 / 3000 [ 23%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  800 / 3000 [ 26%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  900 / 3000 [ 30%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1000 / 3000 [ 33%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1001 / 3000 [ 33%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1100 / 3000 [ 36%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1200 / 3000 [ 40%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1300 / 3000 [ 43%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1400 / 3000 [ 46%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1500 / 3000 [ 50%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1600 / 3000 [ 53%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1700 / 3000 [ 56%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1800 / 3000 [ 60%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1900 / 3000 [ 63%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2000 / 3000 [ 66%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2100 / 3000 [ 70%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2200 / 3000 [ 73%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2300 / 3000 [ 76%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2400 / 3000 [ 80%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2500 / 3000 [ 83%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2600 / 3000 [ 86%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2700 / 3000 [ 90%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2800 / 3000 [ 93%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2900 / 3000 [ 96%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 3000 / 3000 [100%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info: Elapsed Time: 0.062 seconds (Warm-up)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.126 seconds (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.188 seconds (Total)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"initialization\\\",\\\"values\\\":[-0.15819898881256257,-1.5319420582368118,-0.28985475046909806]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Adaptation terminated\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Step size = 0.0677414\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Diagonal elements of inverse mass matrix:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"0.26831, 0.007586\" <> ...],\n  num_chains: 4,\n  param_names: [\"alpha\", \"beta\", \"sigma\"],\n  feature_names: [\"accept_stat__\", \"alpha\", \"beta\", \"divergent__\", \"energy__\", \"lp__\",\n   \"n_leapfrog__\", \"sigma\", \"stepsize__\", \"treedepth__\"],\n  constrained_param_names: [\"alpha\", \"beta\", \"sigma\"],\n  dims: [[], [], []],\n  num_warmup: 1000,\n  num_samples: 2000,\n  num_thin: 1,\n  num_flat: 1,\n  save_warmup: false,\n  sample_and_sampler_param_names: [\"accept_stat__\", \"divergent__\", \"energy__\", \"lp__\",\n   \"n_leapfrog__\", \"stepsize__\", \"treedepth__\"],\n  draws: #Nx.Tensor \n}\n```","ref":"iris_linear_regression.html#inference"},{"type":"extras","title":"Evaluation - Iris: Linear Regression","doc":"#","ref":"iris_linear_regression.html#evaluation"},{"type":"extras","title":"Traceplot - Iris: Linear Regression","doc":"```elixir\nfit_df = Fit.to_frame(fit)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```text\n#Explorer.DataFrame \n```\n\n```elixir\nparam_list = fit_df |> DF.select([\"alpha\", \"sample_number\", \"chain_number\"]) |> DF.to_rows()\n\nVl.new(width: 700, height: 400)\n|> Vl.data_from_values(param_list)\n|> Vl.mark(:line)\n|> Vl.encode_field(:x, \"sample_number\", type: :quantitative)\n|> Vl.encode_field(:y, \"alpha\", type: :quantitative)\n|> Vl.encode_field(:color, \"chain_number\", type: :nominal, scale: [scheme: \"blues\"])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n\n\nThe above diagram shows that all chains have mixed well with each other i.e they've all arrived at a similar stationary distribution, showing that the model has converged.\n\n<!-- livebook:{\"break_markdown\":true} -->\n\n#","ref":"iris_linear_regression.html#traceplot"},{"type":"extras","title":"Summary Statistics - Iris: Linear Regression","doc":"```elixir\nDF.describe(fit_df, percentiles: [0.05, 0.5, 0.95])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```text\n#Explorer.DataFrame \n```\n\nThe above parameters can be explained as follows:\n\n* **lp__**: Log probability representing the log of the posterior density; used for debugging and model comparison.\n* **accept_stat__**: Acceptance probability of proposed samples in HMC/NUTS; indicates sampling efficiency.\n* **stepsize__**: Step size used in numerical integration of Hamiltonian dynamics; controls resolution of exploration.\n* **treedepth__**: Depth of tree built during each NUTS iteration; indicates exploration depth.\n* **n_leapfrog__**: Number of leapfrog steps taken during HMC/NUTS integration; indicates exploration efficiency.\n* **divergent__**: Binary indicator showing if an iteration experienced divergence; indicates numerical integration issues.\n* **energy__**: Hamiltonian energy of the system used by NUTS; indicates exploration behavior.\n\n<!-- livebook:{\"break_markdown\":true} -->\n\n#","ref":"iris_linear_regression.html#summary-statistics"},{"type":"extras","title":"Parameter Plot - Iris: Linear Regression","doc":"```elixir\ntmp = DF.select(fit_df, fit.param_names) |> DF.to_rows()\n\nnew_list =\n  Enum.flat_map(tmp, fn map ->\n    Enum.map(map, fn {key, value} ->\n      %{\n        \"param_name\" => key,\n        \"param_value\" => value\n      }\n    end)\n  end)\n\nVl.new(width: 800, height: 300)\n|> Vl.data_from_values(new_list)\n|> Vl.encode_field(:y, \"param_name\", type: :ordinal, title: \"Parameter Name\")\n|> Vl.layers([\n  Vl.new()\n  |> Vl.mark(:point, filled: true)\n  |> Vl.encode_field(:x, \"param_value\",\n    aggregate: :mean,\n    type: :quantitative,\n    scale: [zero: false]\n  )\n  |> Vl.encode(:color, value: :black),\n  Vl.new()\n  |> Vl.mark(:errorbar, extent: :stdev)\n  |> Vl.encode_field(:x, \"param_value\", type: :quantitative, title: \"Parameter Value\")\n])\n```\n\n<!-- livebook:{\"output\":true} -->","ref":"iris_linear_regression.html#parameter-plot"},{"type":"extras","title":"Posterior Plot - Iris: Linear Regression","doc":"```elixir\nkey = Nx.Random.key(42)\n\nalpha_mean =\n  fit_df\n  |> DF.select([\"alpha\"])\n  |> DF.to_series()\n  |> Map.get(\"alpha\")\n  |> Explorer.Series.mean()\n\nbeta_mean =\n  fit_df\n  |> DF.select([\"beta\"])\n  |> DF.to_series()\n  |> Map.get(\"beta\")\n  |> Explorer.Series.mean()\n\nsigma_mean =\n  fit_df\n  |> DF.select([\"sigma\"])\n  |> DF.to_series()\n  |> Map.get(\"sigma\")\n  |> Explorer.Series.mean()\n\nx_tensor = Nx.tensor(x)\ny_tensor = Nx.tensor(y)\npredicted = Nx.multiply(beta_mean, x_tensor) |> Nx.add(alpha_mean)\n\npredicted_list = Nx.to_list(predicted)\n\nresult =\n  DF.new(\n    sepal_length: x,\n    petal_length: y,\n    predicted: predicted_list\n  )\n  |> DF.to_rows()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  %{\"petal_length\" => 4.7, \"predicted\" => 4.988199710845947, \"sepal_length\" => 7.0},\n  %{\"petal_length\" => 4.5, \"predicted\" => 4.577447891235352, \"sepal_length\" => 6.4},\n  %{\"petal_length\" => 4.9, \"predicted\" => 4.919741153717041, \"sepal_length\" => 6.9},\n  %{\"petal_length\" => 4.0, \"predicted\" => 3.961320161819458, \"sepal_length\" => 5.5},\n  %{\"petal_length\" => 4.6, \"predicted\" => 4.645906448364258, \"sepal_length\" => 6.5},\n  %{\"petal_length\" => 4.5, \"predicted\" => 4.09823751449585, \"sepal_length\" => 5.7},\n  %{\"petal_length\" => 4.7, \"predicted\" => 4.508989334106445, \"sepal_length\" => 6.3},\n  %{\"petal_length\" => 3.3, \"predicted\" => 3.5505685806274414, \"sepal_length\" => 4.9},\n  %{\"petal_length\" => 4.6, \"predicted\" => 4.714365005493164, \"sepal_length\" => 6.6},\n  %{\"petal_length\" => 3.9, \"predicted\" => 3.75594425201416, \"sepal_length\" => 5.2},\n  %{\"petal_length\" => 3.5, \"predicted\" => 3.6190271377563477, \"sepal_length\" => 5.0},\n  %{\"petal_length\" => 4.2, \"predicted\" => 4.235154628753662, \"sepal_length\" => 5.9},\n  %{\"petal_length\" => 4.0, \"predicted\" => 4.303613185882568, \"sepal_length\" => 6.0},\n  %{\"petal_length\" => 4.7, \"predicted\" => 4.372071743011475, \"sepal_length\" => 6.1},\n  %{\"petal_length\" => 3.6, \"predicted\" => 4.029778957366943, \"sepal_length\" => 5.6},\n  %{\"petal_length\" => 4.4, \"predicted\" => 4.78282356262207, \"sepal_length\" => 6.7},\n  %{\"petal_length\" => 4.5, \"predicted\" => 4.029778957366943, \"sepal_length\" => 5.6},\n  %{\"petal_length\" => 4.1, \"predicted\" => 4.166696071624756, \"sepal_length\" => 5.8},\n  %{\"petal_length\" => 4.5, \"predicted\" => 4.440530300140381, \"sepal_length\" => 6.2},\n  %{\"petal_length\" => 3.9, \"predicted\" => 4.029778957366943, \"sepal_length\" => 5.6},\n  %{\"petal_length\" => 4.8, \"predicted\" => 4.235154628753662, \"sepal_length\" => 5.9},\n  %{\"petal_length\" => 4.0, \"predicted\" => 4.372071743011475, \"sepal_length\" => 6.1},\n  %{\"petal_length\" => 4.9, \"predicted\" => 4.508989334106445, \"sepal_length\" => 6.3},\n  %{\"petal_length\" => 4.7, \"predicted\" => 4.372071743011475, \"sepal_length\" => 6.1},\n  %{\"petal_length\" => 4.3, \"predicted\" => 4.577447891235352, \"sepal_length\" => 6.4},\n  %{\"petal_length\" => 4.4, \"predicted\" => 4.714365005493164, \"sepal_length\" => 6.6},\n  %{\"petal_length\" => 4.8, \"predicted\" => 4.851282596588135, \"sepal_length\" => 6.8},\n  %{\"petal_length\" => 5.0, \"predicted\" => 4.78282356262207, \"sepal_length\" => 6.7},\n  %{\"petal_length\" => 4.5, \"predicted\" => 4.303613185882568, \"sepal_length\" => 6.0},\n  %{\"petal_length\" => 3.5, \"predicted\" => 4.09823751449585, \"sepal_length\" => 5.7},\n  %{\"petal_length\" => 3.8, \"predicted\" => 3.961320161819458, \"sepal_length\" => 5.5},\n  %{\"petal_length\" => 3.7, \"predicted\" => 3.961320161819458, \"sepal_length\" => 5.5},\n  %{\"petal_length\" => 3.9, \"predicted\" => 4.166696071624756, \"sepal_length\" => 5.8},\n  %{\"petal_length\" => 5.1, \"predicted\" => 4.303613185882568, \"sepal_length\" => 6.0},\n  %{\"petal_length\" => 4.5, \"predicted\" => 3.8928616046905518, \"sepal_length\" => 5.4},\n  %{\"petal_length\" => 4.5, \"predicted\" => 4.303613185882568, \"sepal_length\" => 6.0},\n  %{\"petal_length\" => 4.7, \"predicted\" => 4.78282356262207, \"sepal_length\" => 6.7},\n  %{\"petal_length\" => 4.4, \"predicted\" => 4.508989334106445, \"sepal_length\" => 6.3},\n  %{\"petal_length\" => 4.1, \"predicted\" => 4.029778957366943, \"sepal_length\" => 5.6},\n  %{\"petal_length\" => 4.0, \"predicted\" => 3.961320161819458, \"sepal_length\" => 5.5},\n  %{\"petal_length\" => 4.4, \"predicted\" => 3.961320161819458, \"sepal_length\" => 5.5},\n  %{\"petal_length\" => 4.6, \"predicted\" => 4.372071743011475, \"sepal_length\" => 6.1},\n  %{\"petal_length\" => 4.0, \"predicted\" => 4.166696071624756, \"sepal_length\" => 5.8},\n  %{\"petal_length\" => 3.3, \"predicted\" => 3.6190271377563477, \"sepal_length\" => 5.0},\n  %{\"petal_length\" => 4.2, \"predicted\" => 4.029778957366943, \"sepal_length\" => 5.6},\n  %{\"petal_length\" => 4.2, \"predicted\" => 4.09823751449585, \"sepal_length\" => 5.7},\n  %{\"petal_length\" => 4.2, \"predicted\" => 4.09823751449585, \"sepal_length\" => 5.7},\n  %{\"petal_length\" => 4.3, \"predicted\" => 4.440530300140381, ...},\n  %{\"petal_length\" => 3.0, ...},\n  %{...}\n]\n```\n\n```elixir\nVl.new(width: 700, height: 300, title: \"Linear Regression Plot\")\n|> Vl.data_from_values(result)\n|> Vl.encode_field(:x, \"sepal_length\", type: :quantitative, scale: [domain: [4.5, 7.5]])\n|> Vl.layers([\n  Vl.new()\n  |> Vl.mark(:point, shape: :square)\n  |> Vl.encode_field(:y, \"petal_length\", type: :quantitative, scale: [domain: [0.5, 6.5]]),\n  Vl.new()\n  |> Vl.mark(:line)\n  |> Vl.encode_field(:y, \"predicted\", type: :quantitative, aggregate: :mean)\n])\n```\n\n<!-- livebook:{\"output\":true} -->","ref":"iris_linear_regression.html#posterior-plot"},{"type":"extras","title":"Posterior Predictive Checks - Iris: Linear Regression","doc":"Here, we will aim simulate data from our posterior distribution. From Stan's documentation:\n\n> Posterior predictive checks are a way of measuring whether a model does a good job of capturing relevant aspects of the data, such as means, standard deviations, and quantiles (Rubin 1984; Andrew Gelman, Meng, and Stern 1996). Posterior predictive checking works by simulating new replicated data sets based on the fitted model parameters and then comparing statistics applied to the replicated data set with the same statistic applied to the original data set.\n\nWe will aim to do this with Stan, using the `generated_quantities` block.\n\n```elixir\nppc_model_code = ~S\"\"\"\ndata {\n  int  N; // number of points\n  vector[N] x; // sepal length i.e covariate\n  vector[N] y; // petal length i.e variate\n}\n\nparameters {\n  real alpha; // intercept\n  real beta; // slope\n  real  sigma; //scatter\n}\n\nmodel {\n  // priors\n  alpha ~ normal(0, 10);\n  beta ~ normal(0, 10);\n  sigma ~ normal(0, 1);\n\n  // likelihood\n  y ~ normal(alpha + beta * x, sigma); \n}\n\ngenerated quantities {\n  vector[N] y_sim; // simulated data from posterior\n\n  for (i in 1:N)\n      y_sim[i] = normal_rng(alpha + beta * x[i], sigma);\n}\n\"\"\"\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\"data {\\n  int  N; // number of points\\n  vector[N] x; // sepal length i.e covariate\\n  vector[N] y; // petal length i.e variate\\n}\\n\\nparameters {\\n  real alpha; // intercept\\n  real beta; // slope\\n  real  sigma; //scatter\\n}\\n\\nmodel {\\n  // priors\\n  alpha ~ normal(0, 10);\\n  beta ~ normal(0, 10);\\n  sigma ~ normal(0, 1);\\n\\n  // likelihood\\n  y ~ normal(alpha + beta * x, sigma); \\n}\\n\\ngenerated quantities {\\n  vector[N] y_sim; // simulated data from posterior\\n\\n  for (i in 1:N)\\n      y_sim[i] = normal_rng(alpha + beta * x[i], sigma);\\n}\\n\"\n```\n\n```elixir\nppc_model = ExStan.build(ppc_model_code, data)\nppc_fit = Model.sample(ppc_model, num_samples: 1000, num_chains: 4, stepsize: 0.1)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n13:09:17.904 [info] Building model..\n\n13:09:17.904 [debug] POST http://localhost:8080/v1/models\n\n13:09:17.910 [info] Model created\n\n13:09:17.910 [info] Messages from stanc: \n\n13:09:17.910 [debug] POST http://localhost:8080/v1/models/4fz34g6a/params\n\n13:09:17.913 [info] `chain` id is set automatically.\n\n13:09:17.913 [debug] POST http://localhost:8080/v1/models/4fz34g6a/fits\n\n13:09:17.916 [debug] POST http://localhost:8080/v1/models/4fz34g6a/fits\n\n13:09:17.934 [debug] POST http://localhost:8080/v1/models/4fz34g6a/fits\n\n13:09:18.188 [debug] POST http://localhost:8080/v1/models/4fz34g6a/fits\n\n13:09:18.362 [debug] GET http://localhost:8080/v1/operations/z5ufub2z\n\n13:09:18.538 [debug] GET http://localhost:8080/v1/operations/7oyoubhj\n\n13:09:18.538 [debug] GET http://localhost:8080/v1/operations/fnpqspp4\n\n13:09:18.539 [debug] GET http://localhost:8080/v1/operations/443nrna2\n\n13:09:18.540 [info] Sampling: 100%, done.\n\n13:09:18.540 [debug] GET http://localhost:8080/v1/models/4fz34g6a/fits/z5ufub2z\n\n13:09:18.555 [debug] DELETE http://localhost:8080/v1/models/4fz34g6a/fits/z5ufub2z\n\n13:09:18.556 [debug] GET http://localhost:8080/v1/models/4fz34g6a/fits/7oyoubhj\n\n13:09:18.588 [debug] DELETE http://localhost:8080/v1/models/4fz34g6a/fits/7oyoubhj\n\n13:09:18.589 [debug] GET http://localhost:8080/v1/models/4fz34g6a/fits/fnpqspp4\n\n13:09:18.607 [debug] DELETE http://localhost:8080/v1/models/4fz34g6a/fits/fnpqspp4\n\n13:09:18.609 [debug] GET http://localhost:8080/v1/models/4fz34g6a/fits/443nrna2\n\n13:09:18.633 [debug] DELETE http://localhost:8080/v1/models/4fz34g6a/fits/443nrna2\n\n13:09:18.679 [info] Messages received during sampling:\n\n13:09:18.680 [info]   Gradient evaluation took 6e-06 seconds\n\n13:09:18.680 [info]   1000 transitions using 10 leapfrog steps per transition would take 0.06 seconds.\n\n13:09:18.680 [info]   Adjust your expectations accordingly!\n\n13:09:18.680 [info]   Gradient evaluation took 5e-06 seconds\n\n13:09:18.680 [info]   1000 transitions using 10 leapfrog steps per transition would take 0.05 seconds.\n\n13:09:18.680 [info]   Adjust your expectations accordingly!\n\n13:09:18.680 [info]   Gradient evaluation took 3e-06 seconds\n\n13:09:18.680 [info]   1000 transitions using 10 leapfrog steps per transition would take 0.03 seconds.\n\n13:09:18.680 [info]   Adjust your expectations accordingly!\n\n13:09:18.680 [info]   Gradient evaluation took 4e-06 seconds\n\n13:09:18.680 [info]   1000 transitions using 10 leapfrog steps per transition would take 0.04 seconds.\n\n13:09:18.680 [info]   Adjust your expectations accordingly!\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%ExStan.Fit{\n  stan_outputs: [\"{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Gradient evaluation took 6e-06 seconds\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:1000 transitions using 10 leapfrog steps per transition would take 0.06 seconds.\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Adjust your expectations accordingly!\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:    1 / 2000 [  0%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  100 / 2000 [  5%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  200 / 2000 [ 10%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  300 / 2000 [ 15%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  400 / 2000 [ 20%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  500 / 2000 [ 25%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  600 / 2000 [ 30%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  700 / 2000 [ 35%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  800 / 2000 [ 40%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  900 / 2000 [ 45%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1000 / 2000 [ 50%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1001 / 2000 [ 50%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1100 / 2000 [ 55%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1200 / 2000 [ 60%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1300 / 2000 [ 65%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1400 / 2000 [ 70%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1500 / 2000 [ 75%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1600 / 2000 [ 80%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1700 / 2000 [ 85%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1800 / 2000 [ 90%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1900 / 2000 [ 95%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2000 / 2000 [100%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info: Elapsed Time: 0.091 seconds (Warm-up)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.163 seconds (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.254 seconds (Total)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"initialization\\\",\\\"values\\\":[-0.9783659596645611,-0.5573192648261118,1.4981911745129347]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Adaptation terminated\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Step size = 0.0576725\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Diagonal elements of inverse mass matrix:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"0.236142, 0.00676247, 0.0112447\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":{\\\"lp__\\\":32.97369668557888,\\\"accept_stat__\\\":0.9969641363127649,\\\"stepsize__\\\":0.057672507174732188,\\\"treedepth__\\\":4.0,\\\"n_leapfrog__\\\":27.0,\\\"divergent__\\\":0.0,\\\"energy__\\\":-29.677431615610318,\\\"alpha\\\":0.005119441075884412,\\\"beta\\\":0.7153146994410063,\\\"sigma\\\":0.31046424650747547,\\\"y_sim.1\\\":4.82349759944606,\\\"y_sim.2\\\":4.7389115294472889,\\\"y_sim.3\\\":4.620237197295994,\\\"y_sim.4\\\":3.1115716923192475,\\\"y_sim.5\\\":4.852241530178888,\\\"y_sim.6\\\":3.8406128903727887,\\\"y_sim.7\\\":4.078493293366337,\\\"y_sim.8\\\":3.1633729835238579,\\\"y_sim.9\\\":4.542629373638011,\\\"y_sim.10\\\":3.346443011570357,\\\"y_sim.11\\\":3.6862971302939329,\\\"y_sim.12\\\":3.659597155166908,\\\"y_sim.13\\\":4.302844716876097,\\\"y_sim.14\\\":4.263521303526952,\\\"y_sim.15\\\":3.869114558043055,\\\"y_sim.16\\\":4.609311522130971,\\\"y_sim.17\\\":4.182280288201898,\\\"y_sim.18\\\":4.598414093801347,\\\"y_sim.19\\\":4.210564131153034,\\\"y_sim.20\\\":3.8367378032274699,\\\"y_sim.21\\\":4.33\" <> ...,\n   \"{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Gradient evaluation took 5e-06 seconds\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:1000 transitions using 10 leapfrog steps per transition would take 0.05 seconds.\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Adjust your expectations accordingly!\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:    1 / 2000 [  0%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  100 / 2000 [  5%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  200 / 2000 [ 10%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  300 / 2000 [ 15%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  400 / 2000 [ 20%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  500 / 2000 [ 25%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  600 / 2000 [ 30%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  700 / 2000 [ 35%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  800 / 2000 [ 40%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  900 / 2000 [ 45%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1000 / 2000 [ 50%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1001 / 2000 [ 50%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1100 / 2000 [ 55%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1200 / 2000 [ 60%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1300 / 2000 [ 65%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1400 / 2000 [ 70%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1500 / 2000 [ 75%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1600 / 2000 [ 80%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1700 / 2000 [ 85%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1800 / 2000 [ 90%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1900 / 2000 [ 95%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2000 / 2000 [100%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info: Elapsed Time: 0.076 seconds (Warm-up)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.07 seconds (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.146 seconds (Total)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"initialization\\\",\\\"values\\\":[0.05442034671090079,-1.4899873790046734,0.10489923554534775]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Adaptation terminated\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Step size = 0.063929\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Diagonal elements of inverse mass matrix:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"0.304065, 0.00841584, 0.0102994\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":{\\\"lp__\\\":32.46995470330302,\\\"accept_stat__\\\":0.8868873265432437,\\\"stepsize__\\\":0.06392903555262691,\\\"treedepth__\\\":4.0,\\\"n_leapfrog__\\\":23.0,\\\"divergent__\\\":0.0,\\\"energy__\\\":-31.73574304675399,\\\"alpha\\\":0.20424749330827958,\\\"beta\\\":0.6910574313443162,\\\"sigma\\\":0.3186003043255849,\\\"y_sim.1\\\":5.122578650522407,\\\"y_sim.2\\\":4.562051644389223,\\\"y_sim.3\\\":4.810301266037838,\\\"y_sim.4\\\":4.7003993119269549,\\\"y_sim.5\\\":3.9059712573175,\\\"y_sim.6\\\":3.6856157258111748,\\\"y_sim.7\\\":4.945223356827866,\\\"y_sim.8\\\":4.070591941498224,\\\"y_sim.9\\\":5.241796515864197,\\\"y_sim.10\\\":3.421046060846399,\\\"y_sim.11\\\":3.800776131898477,\\\"y_sim.12\\\":3.9975353834629865,\\\"y_sim.13\\\":4.0370073083689859,\\\"y_sim.14\\\":4.550947317161923,\\\"y_sim.15\\\":3.8164819953047308,\\\"y_sim.16\\\":4.554120850219271,\\\"y_sim.17\\\":4.37459603470757,\\\"y_sim.18\\\":3.993306958736087,\\\"y_sim.19\\\":4.598908678847114,\\\"y_sim.20\\\":3.80328831588541,\\\"y_sim.21\\\":4.73079062685\" <> ...,\n   \"{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Gradient evaluation took 3e-06 seconds\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:1000 transitions using 10 leapfrog steps per transition would take 0.03 seconds.\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Adjust your expectations accordingly!\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:    1 / 2000 [  0%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  100 / 2000 [  5%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  200 / 2000 [ 10%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  300 / 2000 [ 15%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  400 / 2000 [ 20%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  500 / 2000 [ 25%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  600 / 2000 [ 30%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  700 / 2000 [ 35%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  800 / 2000 [ 40%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  900 / 2000 [ 45%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1000 / 2000 [ 50%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1001 / 2000 [ 50%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1100 / 2000 [ 55%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1200 / 2000 [ 60%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1300 / 2000 [ 65%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1400 / 2000 [ 70%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1500 / 2000 [ 75%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1600 / 2000 [ 80%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1700 / 2000 [ 85%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1800 / 2000 [ 90%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1900 / 2000 [ 95%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2000 / 2000 [100%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info: Elapsed Time: 0.059 seconds (Warm-up)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.074 seconds (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.133 seconds (Total)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"initialization\\\",\\\"values\\\":[-0.2951205304732387,-0.9470579910310857,0.20042005425138632]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Adaptation terminated\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Step size = 0.0590723\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Diagonal elements of inverse mass matrix:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"0.357472, 0.0101886, 0.0105952\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":{\\\"lp__\\\":31.588690693378685,\\\"accept_stat__\\\":1.0,\\\"stepsize__\\\":0.059072263089433637,\\\"treedepth__\\\":1.0,\\\"n_leapfrog__\\\":1.0,\\\"divergent__\\\":0.0,\\\"energy__\\\":-30.7418924218134,\\\"alpha\\\":0.06963091062628729,\\\"beta\\\":0.6937046784631527,\\\"sigma\\\":0.3328321554878919,\\\"y_sim.1\\\":4.6533026028622059,\\\"y_sim.2\\\":4.602971776826893,\\\"y_sim.3\\\":4.844395404327267,\\\"y_sim.4\\\":3.6324321682801368,\\\"y_sim.5\\\":4.460097409964932,\\\"y_sim.6\\\":4.187836119198266,\\\"y_sim.7\\\":4.222040339521549,\\\"y_sim.8\\\":3.5523426866639324,\\\"y_sim.9\\\":4.033565024419359,\\\"y_sim.10\\\":3.9136923756645567,\\\"y_sim.11\\\":3.4333822211416544,\\\"y_sim.12\\\":3.951062895791963,\\\"y_sim.13\\\":4.2857153004457209,\\\"y_sim.14\\\":4.17831241268849,\\\"y_sim.15\\\":4.482991167999929,\\\"y_sim.16\\\":4.274668566549412,\\\"y_sim.17\\\":4.194775080301997,\\\"y_sim.18\\\":3.7610101594058897,\\\"y_sim.19\\\":4.082780246524915,\\\"y_sim.20\\\":4.456824283742072,\\\"y_sim.21\\\":4.3272861790492789,\\\"y_\" <> ...,\n   \"{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Gradient evaluation took 4e-06 seconds\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:1000 transitions using 10 leapfrog steps per transition would take 0.04 seconds.\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Adjust your expectations accordingly!\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:    1 / 2000 [  0%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  100 / 2000 [  5%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  200 / 2000 [ 10%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  300 / 2000 [ 15%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  400 / 2000 [ 20%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  500 / 2000 [ 25%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  600 / 2000 [ 30%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  700 / 2000 [ 35%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  800 / 2000 [ 40%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration:  900 / 2000 [ 45%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1000 / 2000 [ 50%]  (Warmup)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1001 / 2000 [ 50%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1100 / 2000 [ 55%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1200 / 2000 [ 60%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1300 / 2000 [ 65%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1400 / 2000 [ 70%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1500 / 2000 [ 75%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1600 / 2000 [ 80%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1700 / 2000 [ 85%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1800 / 2000 [ 90%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 1900 / 2000 [ 95%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:Iteration: 2000 / 2000 [100%]  (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info: Elapsed Time: 0.063 seconds (Warm-up)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.069 seconds (Sampling)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:               0.132 seconds (Total)\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"logger\\\",\\\"values\\\":[\\\"info:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"initialization\\\",\\\"values\\\":[-0.15819898881256257,-1.5319420582368118,-0.28985475046909806]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Adaptation terminated\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Step size = 0.0677414\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"Diagonal elements of inverse mass matrix:\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":[\\\"0.26831, 0.00758626, 0.0111768\\\"]}\\n{\\\"version\\\":1,\\\"topic\\\":\\\"sample\\\",\\\"values\\\":{\\\"lp__\\\":31.57164404707967,\\\"accept_stat__\\\":0.9234753652457923,\\\"stepsize__\\\":0.06774140344667262,\\\"treedepth__\\\":5.0,\\\"n_leapfrog__\\\":63.0,\\\"divergent__\\\":0.0,\\\"energy__\\\":-29.688534131874499,\\\"alpha\\\":-0.15156067984921826,\\\"beta\\\":0.7515841376160921,\\\"sigma\\\":0.28360737467769289,\\\"y_sim.1\\\":5.0310322096736999,\\\"y_sim.2\\\":4.376444469663497,\\\"y_sim.3\\\":5.264353406422264,\\\"y_sim.4\\\":4.184760035226238,\\\"y_sim.5\\\":4.785879601986433,\\\"y_sim.6\\\":4.378459342566148,\\\"y_sim.7\\\":4.932531879362846,\\\"y_sim.8\\\":4.074046066892218,\\\"y_sim.9\\\":5.335066468458714,\\\"y_sim.10\\\":3.400329199233303,\\\"y_sim.11\\\":3.4201969428254785,\\\"y_sim.12\\\":4.18596015778274,\\\"y_sim.13\\\":4.066756941312525,\\\"y_sim.14\\\":4.531530668254802,\\\"y_sim.15\\\":4.024982663474163,\\\"y_sim.16\\\":5.118482491140103,\\\"y_sim.17\\\":4.141143215483905,\\\"y_sim.18\\\":4.2079021808234489,\\\"y_sim.19\\\":4.5551355278315139,\\\"y_sim.20\\\":4.142215925554809,\\\"y_sim.21\\\":4.578\" <> ...],\n  num_chains: 4,\n  param_names: [\"alpha\", \"beta\", \"sigma\", \"y_sim\"],\n  feature_names: [\"y_sim.36\", \"sigma\", \"y_sim.11\", \"y_sim.18\", \"energy__\", \"y_sim.26\", \"y_sim.1\",\n   \"y_sim.2\", \"y_sim.39\", \"y_sim.7\", \"y_sim.24\", \"y_sim.23\", \"y_sim.28\", \"y_sim.37\", \"beta\",\n   \"y_sim.31\", \"y_sim.40\", \"y_sim.3\", \"lp__\", \"y_sim.15\", \"y_sim.22\", \"y_sim.34\", \"y_sim.38\",\n   \"y_sim.5\", \"accept_stat__\", \"y_sim.4\", \"y_sim.16\", \"y_sim.21\", \"y_sim.25\", \"y_sim.29\",\n   \"y_sim.27\", \"y_sim.9\", \"y_sim.33\", \"y_sim.17\", \"y_sim.8\", \"y_sim.47\", \"treedepth__\", \"y_sim.44\",\n   \"y_sim.13\", \"y_sim.46\", \"y_sim.19\", \"y_sim.45\", \"y_sim.35\", \"y_sim.43\", \"y_sim.6\", \"y_sim.48\",\n   ...],\n  constrained_param_names: [\"alpha\", \"beta\", \"sigma\", \"y_sim.1\", \"y_sim.2\", \"y_sim.3\", \"y_sim.4\",\n   \"y_sim.5\", \"y_sim.6\", \"y_sim.7\", \"y_sim.8\", \"y_sim.9\", \"y_sim.10\", \"y_sim.11\", \"y_sim.12\",\n   \"y_sim.13\", \"y_sim.14\", \"y_sim.15\", \"y_sim.16\", \"y_sim.17\", \"y_sim.18\", \"y_sim.19\", \"y_sim.20\",\n   \"y_sim.21\", \"y_sim.22\", \"y_sim.23\", \"y_sim.24\", \"y_sim.25\", \"y_sim.26\", \"y_sim.27\", \"y_sim.28\",\n   \"y_sim.29\", \"y_sim.30\", \"y_sim.31\", \"y_sim.32\", \"y_sim.33\", \"y_sim.34\", \"y_sim.35\", \"y_sim.36\",\n   \"y_sim.37\", \"y_sim.38\", \"y_sim.39\", \"y_sim.40\", \"y_sim.41\", \"y_sim.42\", ...],\n  dims: [[], [], [], ~c\"2\"],\n  num_warmup: 1000,\n  num_samples: 1000,\n  num_thin: 1,\n  num_flat: 1,\n  save_warmup: false,\n  sample_and_sampler_param_names: [\"energy__\", \"lp__\", \"accept_stat__\", \"treedepth__\",\n   \"divergent__\", \"n_leapfrog__\", \"stepsize__\"],\n  draws: #Nx.Tensor \n}\n```\n\n```elixir\nppc_fit_df = Fit.to_frame(ppc_fit)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```text\n#Explorer.DataFrame \n```\n\n```elixir\ny_sim_col_names =\n  ppc_fit.feature_names\n  |> Enum.filter(fn x -> String.contains?(x, \"y_sim\") end)\n  |> Enum.sort()\n  |> Enum.take(5)\n\ny_sim_rows = ppc_fit_df |> DF.select(y_sim_col_names) |> DF.to_rows()\n\nflat_data =\n  Enum.flat_map(y_sim_rows, fn map ->\n    Enum.map(map, fn {mkey, mvalue} ->\n      %{\"group\" => mkey, \"value\" => mvalue}\n    end)\n  end)\n\ntdata = Enum.map(y, fn elem -> %{\"group\" => \"y_original\", \"value\" => elem} end)\n\nVl.new(width: 700, height: 300)\n|> Vl.data_from_values(tdata ++ flat_data)\n|> Vl.transform(density: \"value\", groupby: [\"group\"])\n|> Vl.mark(:line)\n|> Vl.encode_field(:x, \"value\", type: :quantitative, title: \"Y Value\")\n|> Vl.encode_field(:y, \"density\", type: :quantitative)\n|> Vl.encode(:color,\n  field: \"group\",\n  type: :nominal,\n  scale: [\n    domain: [\"y_sim.1\", \"y_sim.10\", \"y_sim.11\", \"y_sim.12\", \"y_sim.13\", \"y_original\"],\n    range: [\"#b3e2cd\", \"#fdcdac\", \"#cbd5e8\", \"#f4cae4\", \"#b6f5d9\", \"#000000\"]\n  ]\n)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n\n\nAs we can see, the posterior samples are a very close match to the original dataset, indicating that our model has acheived a good fit.\n\nIf posterior is not a close match to the original dataset, it indicates a problem with the model specification, and the priors chosen for the parameters.","ref":"iris_linear_regression.html#posterior-predictive-checks"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}